@@ -5,49 +5,32 @@
 #include <progress.hpp>
 #include "cpploss.h"
 #include <Rcpp.h>
-#include <math.h>
 #include <Rmath.h>
 #include <iostream>  
 
 // [[Rcpp::export]]
 SEXP  GCPM_cpploss(SEXP default_distr_a,SEXP link_function_a, SEXP S_a,SEXP Sigma_a, SEXP W_a, SEXP PD_a, SEXP PL_a, SEXP calc_rc_a, SEXP loss_thr_a,SEXP seed_a,SEXP max_entries_a){
+  
   NumericMatrix S(S_a), W(W_a),Sigma(Sigma_a);
   NumericVector PD(PD_a), PL(PL_a),seed(seed_a),max_entries(max_entries_a),default_distr(default_distr_a),link_function(link_function_a),calc_rc(calc_rc_a),loss_thr(loss_thr_a);
-
+  List ret;
+  
   bool full=false;
   int N=S.nrow(), NS=S.ncol();
   int NCP=W.nrow(),defaults=0;
-  int print_every=(int)floor(N/100),VaRszen=0,MaxVaRszen=std::min((int)(floor(max_entries(0)/NCP)),1000),inflate=1000,temp_int_1=0, temp_int_2=0;
+  int print_every=(int)(floor((float) (N/100))),inflate=1000;
   double condPD=0,temp=0,temp2=0;
-  
+  std::vector<double> lossszenarios(0);
+  std::vector<std::vector<double> > CPsimlosses(NCP, std::vector<double>(0));
   if(calc_rc(0)==1){
-    temp_int_1=NCP;
-    temp_int_2=MaxVaRszen;
-  }
-  else{
-    temp_int_1=1;
-    temp_int_2=1;
-  }
-  double *lossszenarios=new double[temp_int_2];
-  double *buffer=new double[temp_int_2];
-  double **buffer2=new double *[temp_int_1];
-  double **CPsimlosses=new double *[temp_int_1];
-  for(int i=0;i<temp_int_1;i++){
-    CPsimlosses[i]=new double[temp_int_2];
-    buffer2[i]=new double[temp_int_2];
-  }
-      
-  double *CPlosses=new double[NCP];
-  List ret;
-  NumericVector simlosses(N),state(1);
-  double *DD=new double[NCP];
-  if(link_function(0)==2){
+    lossszenarios.reserve(inflate);
     for(int i=0;i<NCP;i++)
-      DD[i]=R::qnorm(PD[i],0,1,1,0);
+      CPsimlosses[i].reserve(inflate);
   }
-  
+  double *CPlosses=new double[NCP];
+  NumericVector simlosses(N),state(1);
   GetRNGstate();
-  Progress p((int)floor(N/print_every), true);  
+  Progress p((int)floor((float)(N/print_every)), true);  
   if(link_function(0)==1){ //CRP
     for(int n=0;n<N;n++){
       if(n%print_every==0 && n>0){
@@ -78,41 +61,17 @@ SEXP  GCPM_cpploss(SEXP default_distr_a,SEXP link_function_a, SEXP S_a,SEXP Sigm
         }
         if(calc_rc(0)==1 && !full){
           if(simlosses(n)>=loss_thr(0)){
-            VaRszen++;
-            if(VaRszen>MaxVaRszen){
-              if((MaxVaRszen+inflate)*NCP<=max_entries(0)){
-                buffer=new double[MaxVaRszen];
-                buffer2=new double *[NCP];
-                for(int p=0;p<NCP;p++)
-                  buffer2[p]=new double[MaxVaRszen];
-                for(int s=0;s<MaxVaRszen;s++){
-                  buffer[s]=lossszenarios[s];
-                  for(int p=0;p<NCP;p++)
-                    buffer2[p][s]=CPsimlosses[p][s];
-                }
-                MaxVaRszen+=inflate;
-                lossszenarios=new double[MaxVaRszen];
-                CPsimlosses=new double *[NCP];
-                for(int i=0;i<NCP;i++)
-                  CPsimlosses[i]=new double[MaxVaRszen];
-                for(int s=0;s<(VaRszen-1);s++){
-                  lossszenarios[s]=buffer[s];
-                  for(int p=0;p<NCP;p++)
-                    CPsimlosses[p][s]=buffer2[p][s];
-                }
-              }
-              else{
-                full=true;
-              }
-            }
-            if(VaRszen<=MaxVaRszen){
-              lossszenarios[VaRszen-1]=n;
+            if(lossszenarios.capacity()==lossszenarios.size())
+              lossszenarios.reserve(lossszenarios.capacity()+inflate);
+            if(CPsimlosses[0].capacity()==CPsimlosses[0].size()){
               for(int i=0;i<NCP;i++)
-                CPsimlosses[i][VaRszen-1]=CPlosses[i];
-            }
-            else{
-              VaRszen--;
+                CPsimlosses[i].reserve(CPsimlosses[0].capacity()+inflate);
             }
+            lossszenarios.push_back(n);
+            for(int i=0;i<NCP;i++)
+                CPsimlosses[i].push_back(CPlosses[i]);
+            if(lossszenarios.size()>=max_entries(0))
+              full=true;
           }
         }
       }
@@ -122,6 +81,10 @@ SEXP  GCPM_cpploss(SEXP default_distr_a,SEXP link_function_a, SEXP S_a,SEXP Sigm
     }
   }
   else if(link_function(0)==2){ //CM
+    double *DD=new double[NCP];
+    for(int i=0;i<NCP;i++)
+      DD[i]=R::qnorm(PD[i],0,1,1,0);
+    
     for(int n=0;n<N;n++){
       if(n%print_every==0 && n>0){
         p.increment();
@@ -155,43 +118,19 @@ SEXP  GCPM_cpploss(SEXP default_distr_a,SEXP link_function_a, SEXP S_a,SEXP Sigm
             CPlosses[i]=PL(i)*defaults; 
           }
         }
-        if(calc_rc(0)==1  && !full){
+        if(calc_rc(0)==1 && !full){
           if(simlosses(n)>=loss_thr(0)){
-            VaRszen++;
-            if(VaRszen>MaxVaRszen){
-              if((MaxVaRszen+inflate)*NCP<=max_entries(0)){
-                buffer=new double[MaxVaRszen];
-                buffer2=new double *[NCP];
-                for(int p=0;p<NCP;p++)
-                  buffer2[p]=new double[MaxVaRszen];
-                for(int s=0;s<MaxVaRszen;s++){
-                  buffer[s]=lossszenarios[s];
-                  for(int p=0;p<NCP;p++)
-                    buffer2[p][s]=CPsimlosses[p][s];
-                }
-                MaxVaRszen+=inflate;
-                lossszenarios=new double[MaxVaRszen];
-                CPsimlosses=new double *[NCP];
-                for(int i=0;i<NCP;i++)
-                  CPsimlosses[i]=new double[MaxVaRszen];
-                for(int s=0;s<(VaRszen-1);s++){
-                  lossszenarios[s]=buffer[s];
-                  for(int p=0;p<NCP;p++)
-                    CPsimlosses[p][s]=buffer2[p][s];
-                }
-              }
-              else{
-                full=true;
-              }
-            }
-            if(VaRszen<=MaxVaRszen){
-              lossszenarios[VaRszen-1]=n;
+            if(lossszenarios.capacity()==lossszenarios.size())
+              lossszenarios.reserve(lossszenarios.capacity()+inflate);
+            if(CPsimlosses[0].capacity()==CPsimlosses[0].size()){
               for(int i=0;i<NCP;i++)
-                CPsimlosses[i][VaRszen-1]=CPlosses[i];
-            }
-            else{
-              VaRszen--;
+                CPsimlosses[i].reserve(CPsimlosses[0].capacity()+inflate);
             }
+            lossszenarios.push_back(n);
+            for(int i=0;i<NCP;i++)
+                CPsimlosses[i].push_back(CPlosses[i]);
+            if(lossszenarios.size()>=max_entries(0))
+              full=true;
           }
         }
       }
@@ -199,22 +138,23 @@ SEXP  GCPM_cpploss(SEXP default_distr_a,SEXP link_function_a, SEXP S_a,SEXP Sigm
         simlosses(n)=-1;
       }
     }
+    delete[] DD;
   }
   PutRNGstate();
   p.increment();
 
-  NumericMatrix CPsimlossesfinal(NCP,VaRszen);
-  NumericVector lossszenariosfinal(VaRszen);
+  NumericMatrix CPsimlossesfinal(NCP,lossszenarios.size());
+  NumericVector lossszenariosfinal(lossszenarios.size());
   if(calc_rc(0)==1){
     if(full){
-      for(int i=0;i<VaRszen;i++){
+      for(int i=0;i<(int)lossszenarios.size();i++){
         for(int j=0;j<NCP;j++)
           CPsimlossesfinal(j,i)=-1;
         lossszenariosfinal(i)=-1;
       }
     }
     else{
-      for(int i=0;i<VaRszen;i++){
+      for(int i=0;i<(int)lossszenarios.size();i++){
         for(int j=0;j<NCP;j++)
           CPsimlossesfinal(j,i)=CPsimlosses[j][i];
         lossszenariosfinal(i)=lossszenarios[i]+1;
@@ -225,15 +165,7 @@ SEXP  GCPM_cpploss(SEXP default_distr_a,SEXP link_function_a, SEXP S_a,SEXP Sigm
   ret["CPsimlosses"]=CPsimlossesfinal;
   ret["lossszenarios"]=lossszenariosfinal;
   
-  delete[] lossszenarios;
-  delete[] buffer;
   delete[] CPlosses;
-  for(int i=0;i<temp_int_1;i++){
-    delete[] CPsimlosses[i];
-    delete[] buffer2[i];
-  }
-  delete[] CPsimlosses;
-  delete[] buffer2;
   return ret;
 }
 
